# flake8: noqa
from .amcache.amcache import AmCachePlugin
from .bcd.boot_entry_list import BootEntryListPlugin
from .ntuser.installed_programs_ntuser import InstalledProgramsNTUserPlugin
from .ntuser.network_drives import NetworkDrivesPlugin
from .ntuser.persistence import NTUserPersistencePlugin
from .ntuser.shellbags_ntuser import ShellBagNtuserPlugin
from .ntuser.tsclient import TSClientPlugin
from .ntuser.typed_urls import TypedUrlsPlugin
from .ntuser.typed_paths import TypedPathsPlugin
from .ntuser.user_assist import UserAssistPlugin
from .ntuser.winrar import WinRARPlugin
from .ntuser.winscp_saved_sessions import WinSCPSavedSessionsPlugin
from .ntuser.word_wheel_query import WordWheelQueryPlugin
from .sam.local_sid import LocalSidPlugin
from .security.domain_sid import DomainSidPlugin
from .software.classes_installer import SoftwareClassesInstallerPlugin
from .software.image_file_execution_options import ImageFileExecutionOptions
from .software.installed_programs import InstalledProgramsSoftwarePlugin
from .software.last_logon import LastLogonPlugin
from .software.persistence import SoftwarePersistencePlugin
from .software.printdemon import PrintDemonPlugin
from .software.profilelist import ProfileListPlugin
from .software.tracing import RASTracingPlugin
from .software.uac import UACStatusPlugin
from .system.active_controlset import ActiveControlSetPlugin
from .system.bam import BAMPlugin
from .system.bootkey import BootKeyPlugin
from .system.computer_name import ComputerNamePlugin
from .system.host_domain_name import HostDomainNamePlugin
from .system.routes import RoutesPlugin
from .system.safeboot_configuration import SafeBootConfigurationPlugin
from .system.services import ServicesPlugin
from .system.shimcache import ShimCachePlugin
from .system.timezone_data import TimezoneDataPlugin
from .system.usbstor import USBSTORPlugin
from .system.wdigest import WDIGESTPlugin
from .usrclass.shellbags_usrclass import ShellBagUsrclassPlugin
from .ntuser.classes_installer import NtuserClassesInstallerPlugin
from .system.network_data import NetworkDataPlugin
from .software.winver import WinVersionPlugin
from .system.previous_winver import PreviousWinVersionPlugin
from .system.shutdown import ShutdownPlugin
from .system.processor_architecture import ProcessorArchitecturePlugin
from .system.crash_dump import CrashDumpPlugin
from .software.susclient import SusclientPlugin
from .system.disablelastaccess import DisableLastAccessPlugin
from .system.codepage import CodepagePlugin
from .software.disablesr import DisableSRPlugin
from .system.diag_sr import DiagSRPlugin
from .software.spp_clients import SppClientsPlugin
from .system.backuprestore import BackupRestorePlugin
from .system.timezone_data2 import TimezoneDataPlugin2
from .ntuser.wsl import WSLPlugin