NTUSER_HIVE_TYPE = "ntuser"
SYSTEM_HIVE_TYPE = "system"
AMCACHE_HIVE_TYPE = "amcache"
SOFTWARE_HIVE_TYPE = "software"
SAM_HIVE_TYPE = "sam"
SECURITY_HIVE_TYPE = "security"
CLASSES_ROOT_HIVE_TYPE = "classes_root"
BCD_HIVE_TYPE = "bcd"
USRCLASS_HIVE_TYPE = "usrclass"


SUPPORTED_HIVE_TYPES = [
    NTUSER_HIVE_TYPE,
    SYSTEM_HIVE_TYPE,
    AMCACHE_HIVE_TYPE,
    SOFTWARE_HIVE_TYPE,
    SAM_HIVE_TYPE,
    SECURITY_HIVE_TYPE,
    BCD_HIVE_TYPE,
    USRCLASS_HIVE_TYPE,
]


HVLE_TRANSACTION_LOG_MAGIC = b"HvLE"
DIRT_TRANSACTION_LOG_MAGIC = b"DIRT"
